@@ -20,29 +20,19 @@ class Que:
     def __init__(self):
         self.q = Queue()
 
-    def put(self, obj:QMessage, **kwargs):
-        assert isinstance(obj, QMessage)
-        self.q.put(obj, **kwargs)
+    def put(self, msg:QMessage, **kwargs):
+        assert isinstance(msg, QMessage)
+        self.q.put(msg, **kwargs)
 
-    def get(self, **kwargs) -> QMessage:
-        obj = self.q.get(**kwargs)
-        assert isinstance(obj, QMessage)
-        return obj
-
-    # non blocking get
-    def get_if(self) -> Optional[QMessage]:
+    # does not raise Empty exception, but returns None in case no QMessage
+    def get(self,
+            block: bool=                True,
+            timeout: Optional[float]=   None,
+            ) -> Optional[QMessage]:
         try:
-            obj = self.q.get_nowait()
-            assert isinstance(obj, QMessage)
-            return obj
-        except Empty:
-            return None
-
-    def get_timeout(self, timeout:float, **kwargs) -> Optional[QMessage]:
-        try:
-            obj = self.q.get(timeout=timeout, **kwargs)
-            assert isinstance(obj, QMessage)
-            return obj
+            msg = self.q.get(block=block, timeout=timeout)
+            assert isinstance(msg, QMessage)
+            return msg
         except Empty:
             return None
 
@@ -101,7 +91,7 @@ class ExSubprocess(Process, ABC):
     def subprocess_method(self): pass
 
     # when exception occurs, message with exception data is put on the output que
-    def __exception_handle(self, name: str):
+    def __exception_handle(self, name:str):
         msg = QMessage(
             type=   f'ex_{name}, ExSubprocess id: {self.id}, pid: {self.pid}',
             data=   self.id) # returns ID here to allow process identification
