@@ -21,7 +21,8 @@ import math
 import random
 from typing import Optional
 
-from pypaq.pms.base_types import P_VAL, POINT, PSDD
+from pypaq.pms.base import P_VAL, POINT, PSDD
+from pypaq.lipytools.pylogger import get_pylogger
 
 NO_REF = '__NO-REF__'
 
@@ -33,14 +34,23 @@ class PaSpa:
             self,
             psdd : PSDD,            # params space dictionary
             distance_L2=    True,   # sets L1 or L2 distance for PaSpa
-            verb=           0):
+            logger=         None,
+            loglevel=       20):
+
+        if not logger:
+            logger = get_pylogger(
+                name=       'paspa',
+                add_stamp=  True,
+                folder=     None,
+                level=      loglevel)
+        self.logger = logger
 
-        self.verb = verb
         self.__psdd = psdd
         self.axes = sorted(list(self.__psdd.keys()))
         self.L2 = distance_L2
         self.dim = len(self.__psdd)
-        if self.verb>0:  print(f'\n*** PaSpa (dim: {self.dim}) inits..')
+        self.logger.info(f'*** PaSpa ***  inits..')
+        self.logger.info(f'> (dim: {self.dim})')
 
         # resolve axis type and width and some safety checks
         self.__axT = {} # axis type, [list,tuple]_[float,int,diff] list_diff is not allowed
@@ -73,7 +83,7 @@ class PaSpa:
             self.__axW[axis] = pdef[-1] - pdef[0] if tpn != 'diff' else len(pdef) - 1 # range, for diff_tuple - number of elements
 
         self.rdim = self.get_rdim()
-        if self.verb>0:  print(f' > rdim: {self.rdim:.1f}')
+        self.logger.info(f'> rdim: {self.rdim:.1f}')
 
         # width of str(value) for axes, used for str formatting
         self.__str_width = {}
